%Estimate a finite mixture of bivariate normal distributions
%Maximo Camacho and Lola Gadea 2016
function mcmcout=estimate_K1_bi_uniformative(ys)
rng('default'); %fixing seed

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adjust any of the following to control specification desired %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
M=2000;       % Number of draws from posterior
burnin=500;  % Number of draws discarded
mcmcout.mu=zeros(M,2);         % Stores means
mcmcout.sigma=zeros(M,2,2);    % Stores variances
[N,r]=size(ys);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         Independent priors non-hierarchical                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% prior means mu~N(b0,B0inv), b0=mean(ys) B0=sum[(ys-mean(ys))'(ys-mean(ys))]/T
b0=zeros(1,r);               % b0 is the prior of the mean
B0=zeros(r,r);               % B0 is the prior of the covariance matrix
ys2=ys-repmat(b0,N,1);
for it=1:N
    B0=B0+ys2(it,:)'*ys2(it,:);
end

B0=100*eye(r); 
B0inv=inv(B0);  

% prior variances inv(sigma)~W(c0,C0). We folow Bensmail et al. (1997):
% See toolbook page 60 / book page 192  
c0=2;
C0=0.01*eye(r);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Running MCMC algorithm                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sampling starts sigma given mu 
mu1=mean(ys);  % 1x2 vector 

for m=1:(burnin+M)
  % 1) Sample variances given means p. 191. If inv(sigma1)~W[c0,C0] => inv(sigma1)/mu~W[c1,C1]
  %    where c1=c0+0.5*N and C1=C0+0.5*sum((y-mu1)*(y-mu1)')       

  %fprintf('MCMC number %4.0f of %4.0f\n',m,burnin+M);
  c1=c0+N;
  SS=zeros(r,r);
  ys2=ys-repmat(mu1,N,1);
  for it=1:N
      SS=SS+ys2(it,:)'*ys2(it,:);
  end
  C1=C0+SS;   
  % If Y~W[a,A] => wishrnd(C1,c1)/c1 generates A1
  % Since E(Y)=a*inv(A) => c1*inv(wishrnd(C1,c1)/c1) generates Y1          
  sigma1=inv(wishrnd(inv(C1),c1));   % rxr dranws from W(C1,c1)
  sigma1inv=inv(sigma1);                 % rxr dranws of covariance matrix
  
  % 2) Sample means given variances p. 178. If mu~N[b0,B0] => mu/v2~N[b1,B1]
  %    where inv(B1)=inv(B0)+N*inv(v2) and b1=B1*[inv(v2)*N*mean(y)+inv(B0)*b0] 
  B1inv=B0inv+N*sigma1inv; %#ok<MINV>
  B1=inv(B1inv);
  b1=B1*(sigma1inv*N*mean(ys)'+B0inv*b0');  %#ok<MINV> % rx1 vector 
  mu1=mvnrnd(b1,B1);                        % rx1 dranws of means

  % Compute likelihood
  logdetsigma1=log(det(sigma1));
  llh1=-.5*r*(log(2*pi)); 
  eps=ys-repmat(mu1,N,1);
  llh=llh1-.5*logdetsigma1-.5*sum(eps*sigma1inv.*eps,2)';    %#ok<MINV>
  mixlik=sum(llh);
  
  if m>burnin
    mcmcout.mu(m-burnin,:)=mu1;              % Stores means
    mcmcout.sigma(m-burnin,:,:)=sigma1;      % Stores variances
    mcmcout.log_mixlik(m-burnin,:,:)=mixlik;      % Stores variances
  end
end

%Artifitial clustering in one block
mcmcout.S=ones(M,N);
mcmcout.clust.Smap=ones(N,1);
mcmcout.clust.Sident=ones(N,1);
mcmcout.clust.Ssim=ones(N,1);
%mu=mean(mcmcout.mu);
%sigma=squeeze(mean(mcmcout.sigma,1));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Information criteria
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mcmcout.log_mixlik= mcmcout.log_mixlik(~isnan(mcmcout.log_mixlik)); %remove NaN elements
loglikmax=max(mcmcout.log_mixlik);
dk=2+3; %num parameters, r means and 3 covariance matrix
ic.loglikmax=loglikmax;
ic.aic= -2*loglikmax+2*dk;
ic.bic= -2*loglikmax+log(N)*dk;
ic.entropy_ml=NaN;
ic.iclbic_ml= NaN;
mcmcout.ic=ic;

